$Game::OnTouchImmuneTime = 0;

exec("./surf.cs");
exec("./dosh.cs");
exec("./items.cs");
exec("./literally_autism.cs");

exec("./documentStorage.cs");
schedule(0, 0, DocumentStorage, SurfData, "config/server/surfData.db");

BrickTreasureChestData.isSurfTrack = 1;
BrickTreasureChestOpenData.isSurfTrack = 1;

function Surf::buildTrackList() {
	%pattern = "Add-Ons/Surf_*/save.bls";
	$Surf::TrackCount = 0;
   
	for (%file = findFirstFile(%pattern); %file !$= ""; %file = findNextFile(%pattern)) {
		$Surf::Track[$Surf::TrackCount] = %file;
		$Surf::TrackCount++;
	}
}

function Surf::nextTrack() {
	if ($DefaultMinigame.track $= "") {
		Surf::loadTrack(0);
	}
	else if ($Surf::TrackCount > 1) {
		Surf::loadTrack((mFloor($DefaultMinigame.track) + 1) % $Surf::TrackCount);
	}
}

function Surf::loadTrack(%i) {
	$DefaultMinigame.track = %i;
	$Surf::TrackName = "";

	%fileName = $Surf::Track[%i];

	for (%i = 0; %i < $defaultMiniGame.numMembers; %i++) {
		%client = $defaultMiniGame.member[%i];
		%player = %client.player;

		if (isObject(%player)) {
			%player.delete();
		}

		%client.camera.setFlyMode();
		%client.camera.mode = "Observer";

		%client.setControlObject(%client.camera);
	}

	BrickGroup_888888.chainDeleteCallback = "Surf::loadTrackPhase2(\"" @ %fileName @ "\");";
	BrickGroup_888888.chainDeleteAll();
}

function Surf::loadTrackPhase2(%fileName) {
	echo("Loading surf track " @ %fileName);

	%displayName = strReplace(%fileName, "Add-Ons/Surf_", "");
	%displayName = strReplace(%displayName, "/save.bls", "");
	%displayName = strReplace(%displayName, "_", " ");
   
	%message = "\c5Now loading \c6" @ %displayName;
	%creditsFileName = filePath(%fileName) @ "/credits.txt";

	if (isFile(%creditsFileName)) {
		%fp = new FileObject();
		%fp.openforRead(%creditsFileName);

		%credit = stripMLControlChars(%fp.readLine());
		%message = %message @ "\c5, created by \c3" @ %credit;

		%fp.close();
		%fp.delete();
	}

	messageAll('', %message);
	%envFileName = filePath(%fileName) @ "/environment.txt";

	if (isFile(%envFileName)) {
		%res = GameModeGuiServer::ParseGameModeFile(%envFileName, 1);

		EnvGuiServer::getIdxFromFilenames();
		EnvGuiServer::SetSimpleMode();

		if (!$EnvGuiServer::SimpleMode) {
			EnvGuiServer::fillAdvancedVarsFromSimple();
			EnvGuiServer::SetAdvancedMode();
		}
	}

	schedule(16, 0, serverDirectSaveFileLoad, %fileName, 3, "", 2, 1);
}

function serverCmdTrackList(%client) {
	for (%i = 0; %i < $Surf::TrackCount; %i++) {
		%displayName = $Surf::Track[%i];

		%displayName = strReplace(%displayName, "Add-Ons/Surf_", "");
		%displayName = strReplace(%displayName, "/save.bls", "");
		%displayName = strReplace(%displayName, "_", " ");

		if (%i == $DefaultMinigame.track) {
			messageClient(%client, '', " >" @ %i + 1 @ ". \c6" @ %displayName);
		}
		else {
			messageClient(%client, '', "   " @ %i + 1 @ ". \c6" @ %displayName);
		}
	}
}

function serverCmdSetTrack(%client, %i) {
	if (!%client.isAdmin) {
		return;
	}

	if (mFloor(%i) !$= %i) {
		messageClient(%client, '', "Usage: /setTrack <number>");
		return;
	}

	%i--;

	if (%i < 0 || %i > $Surf::TrackCount) {
		messageClient(%client, '', "serverCmdSetTrack() - out of range index");
		return;
	}

	messageAll('MsgAdminForce', '\c3%1\c2 changed the track', %client.getPlayerName());
	Surf::loadTrack(%i);
}

function serverCmdNextTrack(%client) {
	if (!%client.isAdmin) {
		return;
	}

	messageAll('MsgAdminForce', '\c3%1\c2 changed the track', %client.getPlayerName());
	Surf::nextTrack();
}

package GameModeSurfPackage {
	function GameConnection::autoAdminCheck(%this) {
		%this.initSurfItems();
		return Parent::autoAdminCheck(%this);
	}

	function GameConnection::onClientEnterGame(%this) {
		Parent::onClientEnterGame(%this);

		if (isObject(SurfData)) {
			%today = getWord(getDateTime(), 0);
			%user = SurfData.collection("users").document(%this.getBLID());

			if (%user.get("last_play_day") !$= %today) {
				%user.set("last_play_day", %today);
				%this.addDosh(10, "playing on the server today");
			}
		}

		%this.freeDoshTick = %this.schedule(300000, freeDoshTick);
	}

	function GameConnection::spawnPlayer(%this) {
		Parent::spawnPlayer(%this);

		if (isObject(%this.player)) {
			%this.equipSurfItems(%this.player);
		}
	}

	function gameModeInitialResetCheck() {
		Parent::gameModeInitialResetCheck();

		if (!$Surf::TrackCount) {
			Surf::buildTrackList();
		}

		if ($Surf::TrackCount == 0) {
			messageAll('', "\c5No surfing tracks available!");
			return;
		}

		if ($DefaultMinigame.track !$= "") {
			return;
		}

		$DefaultMinigame.track = "";
		Surf::nextTrack();
	}

	function serverLoadSaveFile_End() {
		Parent::serverLoadSaveFile_End();

		if($DefaultMiniGame.numMembers > 0) {
			$DefaultMiniGame.scheduleReset();
		}
	}

	function FxDTSBrick::onActivate(%this, %obj, %client, %position, %vector) {
		if (%obj.isSurfing && %this.getDataBlock().isSurfTrack) {
			%obj.isSurfing = 0;
			%time = getSimTime() - %obj.surfStartTime;

			if (%this.getName() $= "") {
				%hash = sha1(%this.getPosition() @ %this.angleId);
			}
			else {
				%hash = %this.getName();
				%hash = getSubStr(%hash, 1, strLen(%hash));

				%ranked = 1;
			}

			%name = %client.getPlayerName();
			%formatted = getTimeString(mFloatLength(%time / 1000, 2));

			%message = "<bitmap:base/client/ui/ci/star> \c3" @ %name;
			%message = %message SPC "\c6completed" SPC %hash SPC "in \c3" @ %formatted @ "\c6.";

			messageAll('', %message);

			if (%ranked) {
				onRankedSurfFinish(%client, %hash, %time);
			}
		}

		Parent::onActivate(%this, %obj, %client, %position, %vector);
	}

	function FxDTSBrickData::onLoadPlant(%this, %obj) {
		Parent::onLoadPlant(%this, %obj);

		if (%this.isSurfTrack) {
			%obj.schedule(100, checkTreasureChestSurfTrack);
		}
	}
};

activatePackage("GameModeSurfPackage");

function FxDTSBrick::checkTreasureChestSurfTrack(%this) {
	%name = %this.getName();

	if ($Surf::TrackName $= "" && %name !$= "") {
		$Surf::TrackName = getSubStr(%name, 1, strLen(%name));
	}
}