datablock PlayerData(PlayerSurfArmor : PlayerStandardArmor) {
	uiName = "Surf Player";
	isSurfPlayer = 1;

	canJet = false;

	mass = 120;
	boundingBox = "5 5 10.6";

	airControl = 0.5;
	runSurfaceAngle = 20;

	maxForwardSpeed = 15;
	maxBackwardSpeed = 15;
	maxSideSpeed = 200;
};

function PlayerSurfArmor::onNewDataBlock(%this, %obj) {
	%obj.surfTick();
}

function Player::surfTick(%this) {
	cancel(%this.surfTick);

	if (%this.getState() $= "Dead") {
		return;
	}

	%max = 75;
	%min = 25;

	if (!%this.isSurfing && !%this.startedSurfing && getSimTime() - %this.spawnTime >= 500) {
		if (mAbs(getWord(%this.getVelocity(), 2)) >= 0.01) {
			%this.isSurfing = 1;
			%this.startedSurfing = 1;

			%this.surfStartTime = getSimTime();
		}
	}

	if (isObject(%this.client)) {
		%speed = vectorLen(%this.getVelocity());
		%text = "<color:AAAAAA>SPEED <color:FFFFAA>" @ mFloatLength(%speed / 2, 0) SPC "M/S";

		if (%this.isSurfing) {
			%time = getSimTime() - %this.surfStartTime;
			%text = %text NL "<color:AAAAAA>TIME  <color:FFFFAA>" @ getTimeString(mFloatLength(%time / 1000, 2));

			if ($Surf::TrackName !$= "") {
				%track = SurfData.collection("tracks").document($Surf::TrackName);
				%best_time = %track.get("best_time");

				%text = %text NL "      <color:";

				if (%best_time $= "") {
					%text = %text @ "AAFFAA>NO RECORD";
				}
				else {
					if (%time < %best_time) {
						%text = %text @ "77FF77>";
					}
					else {
						%text = %text @ "FF7777>";
					}

					%text = %text @ getTimeString(mFloatLength(%best_time / 1000, 2));

					%best_time_holder = %track.get("best_time_holder");
					%document = SurfData.collection("users").document(%best_time_holder, 1);

					if (isObject(%document)) {
						%best_time_holder = %document.get("last_name");
					}
					else {
						%best_time_holder = "BL_ID:" SPC %best_time_holder;
					}

					%text = %text @ "<color:AAAAAA><font:lucida console:16>  (" @ %best_time_holder @ ")";
				}
			}
		}

		%factor = mClampF((%speed - %min) / (%max - %min), 0, 1);

		commandToClient(%this.client, 'SetVignette', 1, %factor SPC %factor SPC %factor SPC %factor);
		commandToClient(%this.client, 'BottomPrint', "<font:lucida console:32>" @ %text, 0.25, 1);
	}

	if (getSimTime() - %this.spawnTime > $Game::PlayerInvulnerabilityTime && getWord(%this.position, 2) < 0.3) {
		if (isObject(%this.client)) {
			%this.client.instantRespawn();
		}
		else {
			%this.kill();
		}

		return;
	}

	%this.surfTick = %this.schedule(50, surfTick);
}

function onRankedSurfFinish(%client, %hash, %time) {
	if (!isObject(%client.miniGame) || %client.miniGame.owner != 0) {
		return;
	}

	if (!isObject(SurfData)) {
		return;
	}

	%client.addDosh(10, "completing a track");

	%name = %client.getPlayerName();
	%blid = %client.getBLID();

	%user = SurfData.collection("users").document(%blid);
	%track = SurfData.collection("tracks").document(%hash);

	if (!%user.get("completed_" @ %hash)) {
		%client.addDosh(10, "completing a track for their first time");

		%user.set("unique_completed_tracks", %user.get("unique_completed_tracks") + 1);
		%user.set("completed_" @ %hash, 1);
	}

	%user.set("last_name", %name);
	%user.set("total_completed_tracks", %user.get("total_completed_tracks") + 1);

	%completers = %track.get("completers");

	if (strPos(" " @ %completers @ " ", " " @ %blid @ " ") < 0) {
		%completers = trim(%completers SPC %blid);
		%added = true;

		%track.set("completers", %completers);
	}

	%best_time = %track.get("best_time");

	if (%best_time $= "" || %time < %best_time) {
		%formatted = getTimeString(mFloatLength(%best_time / 1000, 2));
		%message = " \c7- <bitmap:base/client/ui/ci/trophy> \c3" @ %name;

		if (%best_time !$= "") {
			%best_time_holder = %track.get("best_time_holder");
			%document = SurfData.collection("users").document(%best_time_holder, 1);

			if (isObject(%document)) {
				%best_time_holder = %document.get("last_name");
			}
			else {
				%best_time_holder = "BL_ID:" SPC %best_time_holder;
			}

			%message = %message SPC "\c6beat the track record of \c3" @ %formatted;
			%message = %message SPC "\c6held by \c3" @ %best_time_holder @ "\c6.";

			%client.addDosh(15, "beating the track record");
		}
		else {
			%message = %message SPC "\c6was the first to complete the track!";
			%client.addDosh(5, "being the first to complete a track");
		}

		%track.set("best_time", %time);
		%track.set("best_time_holder", %blid);

		messageAll('', %message);
		schedule(500, 0, serverPlay2D, RewardSound);
	}

	%best_time = %user.get("best_time_" @ %hash);

	if (%best_time $= "" || %time < %best_time) {
		%user.set("best_time_" @ %hash, %time);

		if (!%added) {
			%formatted = getTimeString(mFloatLength(%best_time / 1000, 2));

			%message = " \c7- <bitmap:base/client/ui/ci/trophy> \c3" @ %name;
			%message = %message SPC "\c6beat their personal track record of \c3" @ %formatted @ "\c6.";

			messageAll('', %message);
			%client.addDosh(5, "beating their personal record");
		}
	}
}